"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.pipeMaybeReactFromMemberExpression = exports.isImportDeclaration = exports.getImportNamesPace = exports.getImportDeclaration = exports.getImportDefault = exports.getImport = exports.getCallExpression = void 0;
const tslib_1 = require("tslib");
const t = tslib_1.__importStar(require("@babel/types"));
const Effect = tslib_1.__importStar(require("effect/Effect"));
const Option = tslib_1.__importStar(require("effect/Option"));
const getCallExpression = (x) => {
    return x.pipe(Effect.map(Option.map((a) => t.isCallExpression(a.init) ? Option.some(a.init) : Option.none())));
};
exports.getCallExpression = getCallExpression;
const getImport = (x) => {
    return Effect.sync(() => x.path.isImportSpecifier() ? Option.some(x.path.node) : Option.none());
};
exports.getImport = getImport;
const getImportDefault = (x) => {
    return Effect.sync(() => x.path.isImportDefaultSpecifier() ? Option.some(x.path.node) : Option.none());
};
exports.getImportDefault = getImportDefault;
const getImportDeclaration = (x) => {
    return Effect.sync(() => x.path.isImportDeclaration() ? Option.some(x.path.node) : Option.none());
};
exports.getImportDeclaration = getImportDeclaration;
const getImportNamesPace = (x) => {
    return Effect.sync(() => x.path.isImportNamespaceSpecifier() ? Option.some(x.path.node) : Option.none());
};
exports.getImportNamesPace = getImportNamesPace;
const isImportDeclaration = (x) => {
    return Effect.sync(() => t.isImportDeclaration(x.node));
};
exports.isImportDeclaration = isImportDeclaration;
/**
 * @step `Step #1`
 * @param {t.MemberExpression} node
 * @returns
 */
const getCreateElementIdent = (node) => {
    return Option.Do.pipe(() => t.isIdentifier(node.property, { name: 'createElement' })
        ? Option.some(node.object)
        : Option.none());
};
/**
 * @step `Step #2`
 */
const getReactIdent = (node) => {
    return node.pipe(Option.flatMap((x) => t.isIdentifier(x, { name: 'react' }) || t.isIdentifier(x, { name: 'React' })
        ? Option.some(x)
        : Option.none()));
};
/**
 * @step `Step #3`
 */
const getBinding = (path) => (node) => {
    return node.pipe(Option.flatMap((a) => Option.fromNullable(path.scope.getBinding(a.name))));
};
/**
 * @step `Step #3`
 */
const bindingToVariableDeclarator = (binding) => {
    return binding.pipe(Option.flatMap((x) => x.path.isVariableDeclarator() ? Option.some(x.path.node) : Option.none()));
};
const getBindingCallExpression = (binding) => {
    return binding.pipe(bindingToVariableDeclarator, Option.flatMap((x) => t.isCallExpression(x.init) ? Option.some(x.init) : Option.none()));
};
/**
 * @step `Step #4`
 */
const getReactRequire = (node) => {
    return node.pipe(Option.flatMap((x) => t.isIdentifier(x.callee, { name: 'require' }) &&
        t.isStringLiteral(x.arguments[0], { value: 'react' })
        ? Option.some(true)
        : Option.none()));
};
const pipeMaybeReactFromMemberExpression = (path) => {
    return Effect.Do.pipe(() => Option.some(path), () => getCreateElementIdent(path.node), getReactIdent, getBinding(path), getBindingCallExpression, getReactRequire, Option.getOrElse(() => false));
};
exports.pipeMaybeReactFromMemberExpression = pipeMaybeReactFromMemberExpression;
//# sourceMappingURL=import.effects.js.map